#import "CommonStructs.h"


/*********************************/
/*     LINE COMMAND              */
/*********************************/


@implementation   LineCommand
{
    UIImage*   mainIcon;
    
    CGPoint    firstPoint;
    CGPoint    secondPoint;
    
    id<ICommandsManager>   Manager;
}

- (id)initWithManager:(id<ICommandsManager>)commMan
{
    self = [super init];
    if (self)
    {
        mainIcon = [UIImage imageNamed:@"lines.png"];
        firstPoint = CGPointMake(-1, -1);
        
        Manager = commMan;
    }
    return self;
}

-(UIImage*)   getCommandIcon
{
    return mainIcon;
}
-(NSString*)  getCommandDescription
{
    return @"Line";
}

-(void)       draw:(CGContextRef)currentContext
{
    if (firstPoint.x > -1 && firstPoint.y>-1)
    {
     CGContextBeginPath(currentContext);
    
     CGContextMoveToPoint(currentContext, firstPoint.x, firstPoint.y);
     CGContextAddLineToPoint(currentContext, secondPoint.x, secondPoint.y);
    
     CGContextStrokePath(currentContext);
    }
}

- (void)touchBegan:(CGPoint)pnt
{
    firstPoint = [[Manager getSnapManager] Snap:pnt];
    
    [[Manager getSnapManager] addPoint:firstPoint];
}
- (void)touchMoved:(CGPoint)pnt
{
    secondPoint = [[Manager getSnapManager] Snap:pnt];
}
- (void)touchEnded:(CGPoint)pnt
{
    sgCObject* pObj = sgCreateLine(firstPoint.x, firstPoint.y, 0, secondPoint.x, secondPoint.y, 0);
    [Drawer ApplyStyleToObject:pObj];
    sgGetScene()->AttachObject(pObj);
    
    [[Manager getSnapManager] addPoint:secondPoint];
    [[Manager getSnapManager] addLineFrom:firstPoint To:secondPoint];
    
   
    SG_VECTOR  dir = {secondPoint.x - firstPoint.x, secondPoint.y - firstPoint.y, 0};
    SG_VECTOR  zDir = {0,0,1};
    
    SG_VECTOR  normDir = sgSpaceMath::VectorsVectorMult(dir, zDir);
    
    [[Manager getSnapManager] addLineFrom:firstPoint To:CGPointMake(firstPoint.x + normDir.x, firstPoint.y + normDir.y)];
    [[Manager getSnapManager] addLineFrom:secondPoint To:CGPointMake(secondPoint.x + normDir.x, secondPoint.y + normDir.y)];


    
    firstPoint = CGPointMake(-1, -1);

}

@end



/*********************************/
/*     CIRCLE COMMAND            */
/*********************************/

@implementation   CircleCommand
{
    UIImage*   mainIcon;
    
    CGPoint    firstPoint;
    CGPoint    secondPoint;

    id<ICommandsManager>   Manager;
}

- (id)initWithManager:(id<ICommandsManager>)commMan
{
    self = [super init];
    if (self)
    {
        mainIcon = [UIImage imageNamed:@"circles.png"];
        
        firstPoint = CGPointMake(-1, -1);
        
        Manager = commMan;

    }
    return self;
}

-(UIImage*)   getCommandIcon
{
    return mainIcon;
}
-(NSString*)  getCommandDescription
{
    return @"Circle";
}

-(void)       draw:(CGContextRef)currentContext
{
    if (firstPoint.x > -1 && firstPoint.y>-1)
    {
      CGFloat xDist = (secondPoint.x-firstPoint.x);
      CGFloat yDist = (secondPoint.y-firstPoint.y);
      CGFloat distance = sqrt((xDist * xDist) + (yDist * yDist));
    
      CGContextBeginPath(currentContext);
    
      CGRect circleRct = CGRectMake(firstPoint.x - distance, firstPoint.y - distance, 2*distance, 2*distance);
      CGContextStrokeEllipseInRect(currentContext, circleRct);
    
      CGContextStrokePath(currentContext);
    }

}

- (void)touchBegan:(CGPoint)pnt
{
    firstPoint = [[Manager getSnapManager] Snap:pnt];
    
    [[Manager getSnapManager] addPoint:firstPoint];
}
- (void)touchMoved:(CGPoint)pnt
{
    secondPoint = [[Manager getSnapManager] Snap:pnt];
}

- (void)touchEnded:(CGPoint)pnt
{
    SG_POINT   crCen = {firstPoint.x, firstPoint.y, 0};
    SG_VECTOR  crNor;
    crNor.x = 0.0;
    crNor.y = 0.0;
    crNor.z = 1.0;
    sgSpaceMath::NormalVector(crNor);
    CGFloat xDist = (secondPoint.x-firstPoint.x);
    CGFloat yDist = (secondPoint.y-firstPoint.y);
    CGFloat distance = sqrt((xDist * xDist) + (yDist * yDist));
    
    SG_CIRCLE  crGeo;
    crGeo.FromCenterRadiusNormal(crCen, distance, crNor);
    sgCCircle* cr = sgCreateCircle(crGeo);
    [Drawer ApplyStyleToObject:cr];

    sgGetScene()->AttachObject(cr);

    firstPoint = CGPointMake(-1, -1);

}


@end






/*********************************/
/*     Arc COMMAND                */
/*********************************/

@implementation   ArcCommand
{
    UIImage*   mainIcon;
    UIImage*   handIcon;
    
    CGPoint    firstPoint;
    CGPoint    secondPoint;
    CGPoint    currentPoint;
    
    CGPoint    handPoint;
    
    BOOL       firstClick;
    
    SG_ARC     arcStruct;
    
    id<ICommandsManager>   Manager;
}

- (id)initWithManager:(id<ICommandsManager>)commMan
{
    self = [super init];
    if (self)
    {
        mainIcon = [UIImage imageNamed:@"arcs.png"];
        handIcon = [UIImage imageNamed:@"hand.png"];

        Manager = commMan;
        
        firstClick = YES;
        
        currentPoint = CGPointMake(-1, -1);
    }
    return self;
}


-(UIImage*)   getCommandIcon
{
    return mainIcon;
}
-(NSString*)  getCommandDescription
{
    return @"Arc";
}

-(void)       draw:(CGContextRef)currentContext
{
    if (currentPoint.x>-1 && currentPoint.y>-1)
    {
        [Drawer DrawArc:&arcStruct inContext:currentContext];
        if (handPoint.x>-1 && handPoint.y>-1)
        {
            CGContextDrawImage(currentContext, CGRectMake(handPoint.x, handPoint.y, handIcon.size.width, handIcon.size.height), handIcon.CGImage);
        }
    }
}

- (void)touchBegan:(CGPoint)pnt
{
    if (firstClick)
    {
        firstPoint = [[Manager getSnapManager] Snap:pnt];
        [[Manager getSnapManager] addPoint:firstPoint];
    }
    else
        currentPoint = pnt;
}
- (void)touchMoved:(CGPoint)pnt
{
    if (firstClick)
    {
        currentPoint = [[Manager getSnapManager] Snap:pnt];
        SG_VECTOR dir = {currentPoint.x - firstPoint.x, currentPoint.y - firstPoint.y, 0};
        SG_VECTOR zAxe = {0,0,1};
        SG_VECTOR norm = sgSpaceMath::VectorsVectorMult(dir, zAxe);
        
        SG_POINT  midPnt = {(currentPoint.x + firstPoint.x)/2 + norm.x/2, (currentPoint.y + firstPoint.y)/2  + norm.y/2, 0};
        
        handPoint = CGPointMake(midPnt.x, midPnt.y);
        
        SG_POINT  fP = {firstPoint.x, firstPoint.y, 0};
        SG_POINT  sP = {currentPoint.x, currentPoint.y, 0};
        
        arcStruct.FromThreePoints(fP, sP, midPnt, false);
    }
    else
    {
        currentPoint = pnt;
        SG_POINT  fP = {firstPoint.x, firstPoint.y, 0};
        SG_POINT  sP = {secondPoint.x, secondPoint.y, 0};
        SG_POINT  curP = {currentPoint.x, currentPoint.y, 0};
        
        SG_VECTOR  aVec = {curP.x - fP.x, curP.y - fP.y, 0};
        SG_VECTOR  bVec = {sP.x - fP.x, sP.y - fP.y, 0};
        
        SG_VECTOR  norm = sgSpaceMath::VectorsVectorMult(aVec, bVec);
        
        if (norm.z>0)
            arcStruct.FromThreePoints(fP, sP, curP, false);
        else
            arcStruct.FromThreePoints(sP, fP, curP, false);
        
        handPoint = CGPointMake(-1, -1);
    }
}
- (void)touchEnded:(CGPoint)pnt
{
    if (firstClick)
    {
        secondPoint = [[Manager getSnapManager] Snap:pnt];
        firstClick = NO;
        [[Manager getSnapManager] addPoint:secondPoint];
    }
    else
    {
        sgCArc* arcOb = sgCreateArc(arcStruct);
        [Drawer ApplyStyleToObject:arcOb];
        sgGetScene()->AttachObject(arcOb);
        
        CGPoint tmpP = CGPointMake(arcStruct.center.x, arcStruct.center.y);
        [[Manager getSnapManager] addPoint:tmpP];

        
        currentPoint = CGPointMake(-1, -1);
        firstClick = YES;

    }
}

@end




/*********************************/
/*     TEXT COMMAND              */
/*********************************/

@implementation   TextCommand
{
    UIImage*   mainIcon;
    
    CGPoint    currentPoint;
    
    id<ICommandsManager>   Manager;
    
    UIAlertView* getTextWindow;
    UITextField* getTextField;

}

- (id)initWithManager:(id<ICommandsManager>)commMan
{
    self = [super init];
    if (self)
    {
        mainIcon = [UIImage imageNamed:@"texts.png"];
        currentPoint = CGPointMake(-1, -1);
        
        Manager = commMan;
        
        getTextWindow = [[UIAlertView alloc] initWithTitle:@"Text" message:@"\n\n" delegate:self cancelButtonTitle:@"Cancel" otherButtonTitles:@"Ok", nil];
        getTextField = [[UITextField alloc] init];
        [getTextField setBackgroundColor:[UIColor whiteColor]];
        getTextField.delegate = self;
        getTextField.borderStyle = UITextBorderStyleLine;
        getTextField.frame = CGRectMake(15, 55, 255, 30);
        getTextField.font = [UIFont fontWithName:@"ArialMT" size:20];
        getTextField.placeholder = nil;
        getTextField.textAlignment = NSTextAlignmentCenter;
        getTextField.keyboardAppearance = UIKeyboardAppearanceAlert;
        [getTextField becomeFirstResponder];
        [getTextWindow addSubview:getTextField];


    }
    return self;
}

-(UIImage*)   getCommandIcon
{
    return mainIcon;
}
-(NSString*)  getCommandDescription
{
    return @"Text";
}

-(void)       draw:(CGContextRef)currentContext
{
    if (currentPoint.x > -1 && currentPoint.y>-1)
    {
         [Drawer DrawText:@"Text" onContext:currentContext fromPoint:currentPoint withAlignToCenter:NO];
    }
}

- (void)touchBegan:(CGPoint)pnt
{
    currentPoint = pnt;
}
- (void)touchMoved:(CGPoint)pnt
{
    currentPoint = pnt;
}
- (void)touchEnded:(CGPoint)pnt
{
    [getTextWindow show];
}

- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex
{
    if ([getTextField.text length] <= 0 || buttonIndex == 0)
    {
         currentPoint = CGPointMake(-1, -1);
         return; //If cancel or 0 length string the string doesn't matter
    }
    
    if (buttonIndex == 1)
     {
         NSString* detailString = getTextField.text;
     
         sgCText*  textO = sgCText::Create(sgFontManager::GetFont(sgFontManager::GetCurrentFont()),	[Drawer GetTextStyle], [detailString cStringUsingEncoding:NSASCIIStringEncoding]);
         if (textO)
         {
             [Drawer ApplyStyleToObject:textO];
             SG_VECTOR  textOrigin = {currentPoint.x, currentPoint.y, 0};
             textO->InitTempMatrix()->Translate(textOrigin);
             textO->ApplyTempMatrix();
             textO->DestroyTempMatrix();
             sgGetScene()->AttachObject(textO);
         }
     
         [Manager redrawViews];
     
         currentPoint = CGPointMake(-1, -1);
     }
}



@end




/*********************************/
/*     DIM COMMAND               */
/*********************************/

@implementation   DimCommand
{
    UIImage*   mainIcon;
    
    CGPoint    firstPoint;
    CGPoint    secondPoint;
    
    id<ICommandsManager>   Manager;
}

- (id)initWithManager:(id<ICommandsManager>)commMan
{
    self = [super init];
    if (self)
    {
        mainIcon = [UIImage imageNamed:@"dims.png"];
        Manager = commMan;
        
        secondPoint = CGPointMake(-1, -1);

    }
    return self;
}

-(UIImage*)   getCommandIcon
{
    return mainIcon;
}
-(NSString*)  getCommandDescription
{
    return @"Dim";
}

-(void)      DrawOrAddToScene:(BOOL)sceneFlag inContext:(CGContextRef)currentContext
{
    NSNumberFormatter *formatter = [[NSNumberFormatter alloc] init];
    [formatter setRoundingMode:NSNumberFormatterRoundHalfDown];
    [formatter setMaximumFractionDigits:0];
    
    CGFloat xDist = (secondPoint.x-firstPoint.x);
    CGFloat yDist = (secondPoint.y-firstPoint.y);
    CGFloat distance = sqrt((xDist * xDist) + (yDist * yDist));
    
    SG_VECTOR dir = {secondPoint.x - firstPoint.x, secondPoint.y - firstPoint.y, 0};
    SG_VECTOR zAxe = {0,0,1};
    SG_VECTOR norm = sgSpaceMath::VectorsVectorMult(dir, zAxe);
    if (!sgSpaceMath::NormalVector(norm))
        return;
    
    CGPoint  midPnt = CGPointMake((secondPoint.x + firstPoint.x)/2 + norm.x*50, (secondPoint.y + firstPoint.y)/2  + norm.y*50);
    
    [Drawer DrawDim:currentContext withType:SG_DT_LINEAR fromPoint:firstPoint toPoint:secondPoint  midPoint:midPnt withLabel:[formatter  stringFromNumber:[NSNumber numberWithDouble:fabs(distance)]] addToScene:sceneFlag];
}

-(void)       draw:(CGContextRef)currentContext
{
    if (secondPoint.x > -1 && secondPoint.y>-1)
    {
        [self DrawOrAddToScene:NO inContext:currentContext];
    }
}

- (void)touchBegan:(CGPoint)pnt
{
    firstPoint = [[Manager getSnapManager] Snap:pnt];
}
- (void)touchMoved:(CGPoint)pnt
{
    secondPoint = [[Manager getSnapManager] Snap:pnt];
}

- (void)touchEnded:(CGPoint)pnt
{
    [self DrawOrAddToScene:YES inContext:nil];

    secondPoint = CGPointMake(-1, -1);

}


@end