#import "CommonStructs.h"

static void swapPoints(SG_POINT& pnt1, SG_POINT& pnt2)
{
    SG_POINT tmpP;
    tmpP = pnt1;
    pnt1 = pnt2;
    pnt2 = tmpP;
}

class  DrawShifts : public SG_USER_DYNAMIC_DATA
{
public:
    DrawShifts(int shift):m_shift(shift)
    {
        
    }
    int          getShift()   const   {return m_shift;};
    
    virtual void Finalize()
    {
        delete this;
    }
private:
    int m_shift;
};


static   SG_TEXT_STYLE    text_style;
static   int              lines_width;
static   int              lines_style;


@implementation   Drawer

+(void)  InitStyle
{
    text_style.state = 0;
    text_style.height= 55;
    text_style.proportions= 100.0;
    text_style.angle= 0.0;
    text_style.horiz_space_proportion= 0.0;
    text_style.vert_space_proportion= 50.0;
    
    lines_style = 0;
    lines_width = 1;
}

+(SG_TEXT_STYLE)  GetTextStyle
{
    return text_style;
}

+(void)           SetTextStyle:(SG_TEXT_STYLE)newTS
{
    text_style = newTS;
}

+(void)           SetLinesWidth:(int)newW
{
    lines_width = newW;
}
+(int)            GetLinesWidth
{
    return lines_width;
}
+(void)           SetLinesStyle:(int)newS
{
    lines_style = newS;
}
+(int)            GetLinesStyle
{
    return lines_style;
}

+(void)    ApplyStyleToObject:(sgCObject*)pObj
{
    pObj->SetAttribute(SG_OA_LINE_THICKNESS, lines_width);
    pObj->SetAttribute(SG_OA_LINE_TYPE, lines_style);
}

+(void)           ApplyStyleToContext:(CGContextRef)context;
{
    [Drawer ApplyStyleToContext:context withWidth:lines_width andStyle:lines_style];
}

+(void)           ApplyStyleToContext:(CGContextRef)context withWidth:(int)linesW andStyle:(int)linesStyle
{
    CGContextSetLineWidth(context, linesW*2);
    
    switch (linesStyle) {
        case 1:
        {
            float dash[2]={10 ,10};
            CGContextSetLineDash(context,0,dash,2);
        }
        break;
        case 2:
        {
            float dash[2]={3 ,3};
            CGContextSetLineDash(context,0,dash,2);
        }
        break;
        default:
        {
            float normal[1]={1};
            CGContextSetLineDash(context,0,normal,0);
        }
        break;
    }

}

+(void) DrawArc:(const SG_ARC*)arcStruct inContext:(CGContextRef) context
{
    
    CGMutablePathRef path = CGPathCreateMutable();
    CGPathAddArc(path, NULL, arcStruct->center.x, arcStruct->center.y, arcStruct->radius, M_PI-arcStruct->begin_angle, M_PI-(arcStruct->begin_angle+arcStruct->angle), 0);
    
    CGContextBeginPath(context);
    CGContextAddPath(context, path);
    CGContextStrokePath(context);
    
    
}

+(void)  DrawDim:(CGContextRef)context withType:(SG_DIMENSION_TYPE)dimType fromPoint:(CGPoint)pntA toPoint:(CGPoint)pntB midPoint:(CGPoint)midP withLabel:(NSString*)string addToScene:(BOOL)sceneFlag
{
    if (fabs(pntA.x-pntB.x)<2 && fabs(pntA.y-pntB.y)<2)
        return;
    
    SG_DIMENSION_STYLE        dim_style;
    
	dim_style.behaviour_type = SG_DBT_PARALLEL;
	dim_style.dimension_line = true;
	dim_style.first_side_line = true;
	dim_style.second_side_line = true;
	dim_style.lug_size = 0.1;
    
	dim_style.automatic_arrows = false;
    
	dim_style.out_first_arrow = false;
	dim_style.first_arrow_style = 1;
    
	dim_style.out_second_arrow= false;
	dim_style.second_arrow_style= 1;
    
	dim_style.arrows_size= 25;
    
	dim_style.text_align = SG_TA_CENTER;
    
	dim_style.text_style.state = 0;
	dim_style.text_style.height= 10;
	dim_style.text_style.proportions= 100.0;
	dim_style.text_style.angle= 0.0;
	dim_style.text_style.horiz_space_proportion= 0.0;
	dim_style.text_style.vert_space_proportion= 50.0;
    
	dim_style.precision = 3;
    
	SG_POINT ppp[3];
    if (fabs(pntA.x-pntB.x)>fabs(pntA.y-pntB.y))
    {
        ppp[0].x = (float)pntB.x; ppp[0].y = (float)pntA.y;    ppp[0].z = 0;
        ppp[1].x = (float)pntA.x; ppp[1].y = (float)pntB.y;    ppp[1].z = 0;
    }
    else
    {
        ppp[0].x = (float)pntA.x; ppp[0].y = (float)pntB.y;    ppp[0].z = 0;
        ppp[1].x = (float)pntB.x; ppp[1].y = (float)pntA.y;    ppp[1].z = 0;
    }
	ppp[2].x = (float)midP.x; ppp[2].y = (float)midP.y;    ppp[2].z = 0;
    
	sgCDimensions* dimO = sgCDimensions::Create(dimType, ppp,	sgFontManager::GetFont(0),dim_style, [string cStringUsingEncoding:NSASCIIStringEncoding]);
    
    if (context!=nil && !sceneFlag)
    {
        const int  lns_cnt = dimO->GetLinesCount();
        const SG_LINE*  lns = dimO->GetLines();

	
        CGContextBeginPath(context);
    
        for (int i=0;i<lns_cnt;i++)
        {
            CGContextMoveToPoint(context, -lns[i].p1.x+(pntA.x+pntB.x), lns[i].p1.y);
            CGContextAddLineToPoint(context,  -lns[i].p2.x+(pntA.x+pntB.x), lns[i].p2.y);
        }
    
        CGContextStrokePath(context);
    }
    
    if (!sceneFlag)
    {
       sgGetScene()->DetachObject(dimO);
       sgDeleteObject(dimO);
    }
    else
    {
        [Drawer ApplyStyleToObject:dimO];
        dimO->SetUserDynamicData(new DrawShifts(pntA.x+pntB.x));
    }
    
}

+(void)  DrawText:(NSString*)string onContext:(CGContextRef)context fromPoint:(CGPoint)origPnt withAlignToCenter:(BOOL)centerAlign
{
    
	sgCText*  textO = sgCText::Create(sgFontManager::GetFont(sgFontManager::GetCurrentFont()),	text_style, [string cStringUsingEncoding:NSASCIIStringEncoding]);
    
    const int  lns_cnt = textO->GetLinesCount();
	const SG_LINE*  lns = textO->GetLines();
	
    CGContextBeginPath(context);
    
    int  centerShift = 0;
    if (centerAlign)
    {
        SG_POINT minP, maxP;
        textO->GetGabarits(minP, maxP);
        centerShift = -(minP.x + maxP.x)/2;
    }
    
    for (int i=0;i<lns_cnt;i++)
    {
        CGContextMoveToPoint(context, origPnt.x + centerShift + lns[i].p1.x, origPnt.y-lns[i].p1.y);
        CGContextAddLineToPoint(context,  origPnt.x + centerShift + lns[i].p2.x, origPnt.y-lns[i].p2.y);
    }
    
    CGContextStrokePath(context);
    
    sgGetScene()->DetachObject(textO);
    sgDeleteObject(textO);
    
}



+(void)  drawObject:(const sgCObject*)pObj inContext:(CGContextRef) context;
{
    [Drawer ApplyStyleToContext:context withWidth:pObj->GetAttribute(SG_OA_LINE_THICKNESS) andStyle:pObj->GetAttribute(SG_OA_LINE_TYPE)];
    switch(pObj->GetType())
    {
        case SG_OT_LINE:
            [Drawer  drawLine:(const sgCLine*)pObj inContext:context];
            break;
        case SG_OT_CIRCLE:
            [Drawer  drawCircle:(const sgCCircle*)pObj inContext:context];
            break;
        case SG_OT_ARC:
            [Drawer  drawArc:(const sgCArc*)pObj inContext:context];
            break;
        case SG_OT_TEXT:
            [Drawer  drawText:(const sgCText*)pObj inContext:context];
            break;
        case SG_OT_DIM:
            [Drawer  drawDimension:(const sgCDimensions*)pObj inContext:context];
            break;
        default:
            break;
    }
}

+(void) drawLine:(const sgCLine*)lineObj inContext:(CGContextRef) context
{
    const SG_LINE*  ln = lineObj->GetGeometry();
	
    CGPoint pbeg = CGPointMake(ln->p1.x, ln->p1.y);  //  2D view - Z=0
    CGPoint pend = CGPointMake(ln->p2.x, ln->p2.y);  //  2D view - Z=0
    
    CGContextBeginPath(context);
    
    CGContextMoveToPoint(context, pbeg.x, pbeg.y);
    CGContextAddLineToPoint(context, pend.x, pend.y);
    
    CGContextStrokePath(context);
    
}

+(void) drawCircle:(const sgCCircle*)cirlceObj inContext:(CGContextRef) context
{
    const SG_CIRCLE*  cirGeo = cirlceObj->GetGeometry();
    
    CGContextBeginPath(context);
    
    CGRect circleRct = CGRectMake(cirGeo->center.x - cirGeo->radius, cirGeo->center.y - cirGeo->radius, 2*cirGeo->radius, 2*cirGeo->radius);
    CGContextStrokeEllipseInRect(context, circleRct);
    
    CGContextStrokePath(context);
    
    
}

+(void) drawArc:(const sgCArc*)arcObj inContext:(CGContextRef) context
{
    const SG_ARC*  arcGeo = arcObj->GetGeometry();
    
    CGMutablePathRef path = CGPathCreateMutable();
    CGPathAddArc(path, NULL, arcGeo->center.x, arcGeo->center.y, arcGeo->radius, M_PI-arcGeo->begin_angle, M_PI-(arcGeo->begin_angle+arcGeo->angle), 0);
    
    CGContextBeginPath(context);
    CGContextAddPath(context, path);
    CGContextStrokePath(context);
    
    
}

+(void)  drawText:(const sgCText*)textO inContext:(CGContextRef)context
{
    const int  lns_cnt = textO->GetLinesCount();
	const SG_LINE*  lns = textO->GetLines();
    
    SG_POINT  minP, maxP;
    ((sgCObject*)textO)->GetGabarits(minP, maxP);
    
    CGContextBeginPath(context);
    
    
    for (int i=0;i<lns_cnt;i++)
    {
        CGContextMoveToPoint(context, lns[i].p1.x, 2*minP.y - lns[i].p1.y);
        CGContextAddLineToPoint(context,  lns[i].p2.x, 2*minP.y - lns[i].p2.y);
    }
    
    CGContextStrokePath(context);
}

+(void)  drawDimension:(const sgCDimensions*)dimO inContext:(CGContextRef)context
{
    const int  lns_cnt = dimO->GetLinesCount();
	const SG_LINE*  lns = dimO->GetLines();
    
    DrawShifts*  drData = (DrawShifts*)(dimO->GetUserDymanicData());
    
    CGContextBeginPath(context);
    
    
    for (int i=0;i<lns_cnt;i++)
    {
        CGContextMoveToPoint(context, -lns[i].p1.x+(drData->getShift()), lns[i].p1.y);
        CGContextAddLineToPoint(context,  -lns[i].p2.x+(drData->getShift()), lns[i].p2.y);
    }
    
    CGContextStrokePath(context);
}




@end
